/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2013, Arnaud Roques
 *
 * Project Info:  http://plantuml.sourceforge.net
 * 
 * This file is part of PlantUML.
 *
 * PlantUML is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PlantUML distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * Original Author:  Arnaud Roques
 * 
 * Revision $Revision: 5183 $
 *
 */
package net.sourceforge.plantuml.svek.image;

import java.awt.geom.Dimension2D;
import java.util.List;

import net.sourceforge.plantuml.ColorParam;
import net.sourceforge.plantuml.Dimension2DDouble;
import net.sourceforge.plantuml.FontParam;
import net.sourceforge.plantuml.ISkinParam;
import net.sourceforge.plantuml.SkinParamUtils;
import net.sourceforge.plantuml.Url;
import net.sourceforge.plantuml.cucadiagram.ILeaf;
import net.sourceforge.plantuml.cucadiagram.PortionShower;
import net.sourceforge.plantuml.graphic.HtmlColor;
import net.sourceforge.plantuml.graphic.StringBounder;
import net.sourceforge.plantuml.graphic.TextBlock;
import net.sourceforge.plantuml.svek.AbstractEntityImage;
import net.sourceforge.plantuml.svek.ShapeType;
import net.sourceforge.plantuml.ugraphic.Shadowable;
import net.sourceforge.plantuml.ugraphic.UGraphic;
import net.sourceforge.plantuml.ugraphic.UGraphicHorizontalLine;
import net.sourceforge.plantuml.ugraphic.UGroup;
import net.sourceforge.plantuml.ugraphic.URectangle;
import net.sourceforge.plantuml.ugraphic.UStroke;

public class EntityImageClass extends AbstractEntityImage {

	final private TextBlock body;
	final private int shield;
	final private EntityImageClassHeader2 header;
	final private List<Url> url;
	final private TextBlock mouseOver;
	final private double roundCorner;

	public EntityImageClass(ILeaf entity, ISkinParam skinParam, PortionShower portionShower) {
		super(entity, skinParam);
		this.roundCorner = skinParam.getRoundCorner();
		this.shield = entity.hasNearDecoration() ? 16 : 0;
		this.body = entity.getBody(portionShower).asTextBlock(FontParam.CLASS_ATTRIBUTE, skinParam);

		header = new EntityImageClassHeader2(entity, skinParam, portionShower);
		this.url = entity.getUrls();
		if (entity.getMouseOver() == null) {
			this.mouseOver = null;
		} else {
			this.mouseOver = entity.getMouseOver().asTextBlock(FontParam.CLASS_ATTRIBUTE, skinParam);
		}

	}

	// private int marginEmptyFieldsOrMethod = 13;

	@Override
	public Dimension2D getDimension(StringBounder stringBounder) {
		final Dimension2D dimHeader = header.getDimension(stringBounder);
		final Dimension2D dimBody = body == null ? new Dimension2DDouble(0, 0) : body.calculateDimension(stringBounder);
		final double width = Math.max(dimBody.getWidth(), dimHeader.getWidth());
		final double height = dimBody.getHeight() + dimHeader.getHeight();
		return new Dimension2DDouble(width, height);
	}

	public void drawU(UGraphic ug, double xTheoricalPosition, double yTheoricalPosition) {
		if (url.size() > 0 && url.get(0).isMember() == false) {
			ug.startUrl(url.get(0));
		}
		drawInternal(ug, xTheoricalPosition, yTheoricalPosition);
		if (mouseOver != null) {
			final UGroup g = ug.createGroup();
			ug.getParam().setBackcolor(SkinParamUtils.getColor(getSkinParam(), ColorParam.classBackground, getStereo()));
			final Dimension2D dim = mouseOver.calculateDimension(ug.getStringBounder());
			final Shadowable rect = new URectangle(dim.getWidth(), dim.getHeight());
			if (getSkinParam().shadowing()) {
				rect.setDeltaShadow(4);
			}

			final HtmlColor classBorder = SkinParamUtils.getColor(getSkinParam(), ColorParam.classBorder, getStereo());
			ug.getParam().setColor(classBorder);
			ug.getParam().setBackcolor(SkinParamUtils.getColor(getSkinParam(), ColorParam.classBackground, getStereo()));

			final double x = xTheoricalPosition + 30;
			final double y = yTheoricalPosition + 30;
			ug.getParam().setStroke(new UStroke(1.5));
			g.draw(x, y, rect);
			ug.getParam().setStroke(new UStroke());
			final UGraphic ug2 = new UGraphicHorizontalLine(ug, x, x + dim.getWidth());
			mouseOver.drawU(ug2, x, y);
			g.close();
		}

		if (url.size() > 0 && url.get(0).isMember() == false) {
			ug.closeAction();
		}
	}

	private void drawInternal(UGraphic ug, double xTheoricalPosition, double yTheoricalPosition) {
		final StringBounder stringBounder = ug.getStringBounder();
		final Dimension2D dimTotal = getDimension(stringBounder);
		final Dimension2D dimHeader = header.getDimension(stringBounder);

		final double widthTotal = dimTotal.getWidth();
		final double heightTotal = dimTotal.getHeight();
		final Shadowable rect = new URectangle(widthTotal, heightTotal, roundCorner, roundCorner);
		if (getSkinParam().shadowing()) {
			rect.setDeltaShadow(4);
		}

		final HtmlColor classBorder = SkinParamUtils.getColor(getSkinParam(), ColorParam.classBorder, getStereo());
		ug.getParam().setColor(classBorder);
		HtmlColor backcolor = getEntity().getSpecificBackColor();
		if (backcolor == null) {
			backcolor = SkinParamUtils.getColor(getSkinParam(), ColorParam.classBackground, getStereo());
		}
		ug.getParam().setBackcolor(backcolor);

		double x = xTheoricalPosition;
		double y = yTheoricalPosition;
		ug.getParam().setStroke(new UStroke(1.5));
		ug.draw(x, y, rect);
		ug.getParam().setStroke(new UStroke());

		ug.getParam().setBackcolor(backcolor);
		header.drawU(ug, x, y, dimTotal.getWidth(), dimHeader.getHeight());

		y += dimHeader.getHeight();

		x = xTheoricalPosition;
		if (body != null) {
			ug.getParam().setBackcolor(backcolor);
			ug.getParam().setColor(classBorder);
			// body.asTextBlock(widthTotal).drawU(ug, x, y);
			final UGraphic ug2 = new UGraphicHorizontalLine(ug, x, x + widthTotal);
			body.drawU(ug2, x, y);
		}
	}

	public ShapeType getShapeType() {
		return ShapeType.RECTANGLE;
	}

	public int getShield() {
		return shield;
	}

}
